/**
 * Creates an instance of CompatibilityChecker
 * @class
 * @name CompatibilityChecker
 * @param object window The window object
 */
var CompatibilityChecker = function(){
    var self = this;

    /** @constant {string} */
    this.EDGE_STRING = "Edge";

    /** @var minimumRequirements $window.minimumRequirements */
    var minimumRequirements;

    /** @var appName $window.navigator.appName */
    var appName;

    /** @var userAgent $window.navigator.userAgent */
    var userAgent;

    /** @var appVersion $window.navigator.appVersion */
    var	appVersion;

    /**
     * Returns the browsers search's regExpression.
     *
     * @return regEx Returns the browsers search's regExpression.
     */
    function getBrowserCapabilitiesRegEx(){
        return /(opera|chrome|crios|chromium|safari|firefox|msie)\/?\s*(\.?\d+(\.\d+)*)/i;
    }

    /**
     * Returns the versions search's regExpression.
     *
     * @return regEx Returns the versions search's regExpression.
     */
    function getVersionRegEx(){
        return /version\/([\.\d]+)/i;
    }

    /**
     * Returns an array containing the client's browser and browser's version when it's Edge.
     *
     * @return array Returns an array containing browser and browser's version when it's Edge.
     */
    function getEdgeCapabilitiesSubstring(){
        return userAgent.indexOf(self.EDGE_STRING) !== -1 && userAgent.substring(userAgent.indexOf(self.EDGE_STRING)).split("/");
    }

    /**
     * Returns an array containing the client's browser and browser's version.
     *
     * @return array Returns an array containing browser and browser's version.
     */
    function buildBrowserAndVersionArrayFromRegExp(){
        var macthResult = userAgent.match(getBrowserCapabilitiesRegEx());
        var temp;
        if(macthResult && (temp = userAgent.match(getVersionRegEx()))){
            macthResult[2] = temp[1];
        }
        return macthResult ? [macthResult[1], macthResult[2]] : [appName, appVersion,'-?'];
    }

    /**
     * Fills the difference index of an array with "0".
     *
     * @param version	 array	Actual version.
     * @param minVersion array	The minimum required version.
     */
    function fillVersionArrayLack(versionArr, minVersionArr){
        var version, lackArray, lack = Math.abs(versionArr.length-minVersionArr.length);
        if(version && lack) lackArray = new Array(lack);
        if(versionArr.length<minVersionArr.length) version = versionArr;
        if(versionArr.length>minVersionArr.length) version = minVersionArr;
        if(lackArray) version.push.apply(version, lackArray.fill("0"));
    }

    /**
     * Returns true if the versionArr has all elements equal or from the first greater
     * then minVersionArr elements.
     *
     * @param version	 array	Actual version.
     * @param minVersion array	The minimum required version.
     *
     * @return boolean Returns true if the versionArr has all elements equal or from the
     * first greater then minVersionArr elements.
     */
    function versionComparator(versionArr, minVersionArr){
        for(var i=0; i<versionArr.length; i++){
            if(parseInt(versionArr[i], 10)>parseInt(minVersionArr[i], 10)) return true;
            if(parseInt(versionArr[i], 10)<parseInt(minVersionArr[i], 10)) return false;
        }
        return true;
    }

    /**
     * Returns an array containing the client's browser and browser's version.
     *
     * @param version	 array	Actual version.
     * @param minVersion array	The minimum required version.
     *
     * @return array Returns an array containing browser and browser's version.
     */
    function isVersionSuitable(version, minVersion){
        var versionArr = version.split(".");
        var minVersionArr = minVersion.split(".");
        fillVersionArrayLack(versionArr, minVersionArr);
        return versionComparator(versionArr, minVersionArr);
    }

    /**
     * Returns an array containing the client's browser and browser's version.
     *
     * @param $window object The window object.
     *
     * @return array Returns an array containing browser and browser's version.
     */
    this.getBrowserAndVersionArray = function($window){
        appName = $window.navigator.appName;
        userAgent = $window.navigator.userAgent;
        appVersion = $window.navigator.appVersion;
        var browserAndVersionArray = getEdgeCapabilitiesSubstring() || buildBrowserAndVersionArrayFromRegExp();
        return browserAndVersionArray;
    };

    /**
     * Receives a object and check for matching browser and minimun version and returns true if
     * it's owner browser is suitable.
     *
     * @param $window         object The window object.
     * @param minRequirements object The configiration of valid browsers
     *
     * @return boolean Returns true if browser and version are suitable and false elseway.
     */
    this.checkRequirements = function($window, minRequirements){
        minimumRequirements = minRequirements;
        var browserAndVersion = self.getBrowserAndVersionArray($window);
        var browserName = browserAndVersion[0];
        var browserVersion = browserAndVersion[1];
        var isCompatible = minimumRequirements[browserName] && isVersionSuitable(browserVersion, minimumRequirements[browserName]);
        $window.isBrowserCompatibleToFramework = isCompatible;
        if(isCompatible){
            $window.onFit($window, this);
        } else {
            $window.onNonFit($window, this);
        }
        return isCompatible;
    };

};

function arrayToObject(previous, element){
    var object = {};
    object[previous] = element;
    return object;
}

function emptyFunction(){}

(function(){
    var checker = new CompatibilityChecker();

    window.onFit = window.onFit || emptyFunction;
    window.onNonFit = window.onNonFit || emptyFunction;
    window.minimumRequirements = window.minimumRequirements === "object" ? window.minimumRequirements : {"Chrome": "62"};

    window.actualConfigurations = function(){
        return checker.getBrowserAndVersionArray(window).reduce(arrayToObject);
    };

    checker.checkRequirements(window, window.minimumRequirements);
}());
